==> sources_fancymenu_fabric_3.8.0_MC_1.19.2.jar: de/keksuccino/fancymenu/mixin/mixins/common/server/MixinLivingEntity.java <==
package de.keksuccino.fancymenu.mixin.mixins.common.server;

import de.keksuccino.fancymenu.networking.PacketHandler;
import de.keksuccino.fancymenu.networking.packets.entities.EntityEventPacket;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_2378;
import net.minecraft.class_2960;
import net.minecraft.class_3218;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_1309.class)
public class MixinLivingEntity {

    /** @reason Broadcast FancyMenu entity death events to connected clients. */
    @Inject(method = "die", at = @At("TAIL"))
    private void after_die_FancyMenu(@NotNull class_1282 damageSource, CallbackInfo info) {
        class_1309 self = (class_1309)(Object)this;
        if (!(self.field_6002 instanceof class_3218 serverLevel)) {
            return;
        }
        if (serverLevel.method_8503() == null) {
            return;
        }
        this.broadcastEntityDeath_FancyMenu(serverLevel, self, damageSource);
    }

    @Unique
    private void broadcastEntityDeath_FancyMenu(@NotNull class_3218 level, @NotNull class_1309 entity, @NotNull class_1282 damageSource) {
        EntityEventPacket packet = new EntityEventPacket();
        packet.event_type = EntityEventPacket.EntityEventType.DEATH;
        class_2960 entityKeyLocation = class_2378.field_11145.method_10221(entity.method_5864());
        packet.entity_key = (entityKeyLocation != null) ? entityKeyLocation.toString() : null;
        packet.entity_uuid = entity.method_5667().toString();
        packet.pos_x = entity.method_23317();
        packet.pos_y = entity.method_23318();
        packet.pos_z = entity.method_23321();
        class_2960 levelLocation = level.method_27983().method_29177();
        packet.level_identifier = (levelLocation != null) ? levelLocation.toString() : null;
        packet.damage_type = this.resolveDamageTypeKey_FancyMenu(damageSource);

        class_1297 killer = this.resolveKillerEntity_FancyMenu(entity, damageSource);
        if (killer != null) {
            packet.killer_name = killer.method_5476().getString();
            packet.killer_uuid = killer.method_5667().toString();
            class_2960 killerKeyLocation = class_2378.field_11145.method_10221(killer.method_5864());
            packet.killer_key = (killerKeyLocation != null) ? killerKeyLocation.toString() : null;
            if (packet.killer_key == null && killer instanceof net.minecraft.class_1657) {
                packet.killer_key = "minecraft:player";
            }
        } else {
            packet.killer_name = null;
            packet.killer_uuid = null;
            packet.killer_key = null;
        }

        PacketHandler.sendToAllFancyMenuClients(level.method_8503(), packet);
    }

    @Unique
    @Nullable
    private class_1297 resolveKillerEntity_FancyMenu(@NotNull class_1309 victim, @Nullable class_1282 damageSource) {
        if (damageSource != null) {
            class_1297 killer = damageSource.method_5529();
            if (killer != null) {
                return killer;
            }
            class_1297 directEntity = damageSource.method_5526();
            if (directEntity != null) {
                return directEntity;
            }
        }
        class_1309 killCredit = victim.method_6124();
        return killCredit != null ? killCredit : null;
    }

    @Unique
    @NotNull
    private String resolveDamageTypeKey_FancyMenu(@Nullable class_1282 damageSource) {
        if (damageSource == null) {
            return "unknown";
        }
        String messageId = damageSource.method_5525();
        if (messageId == null || messageId.isEmpty()) {
            return "unknown";
        }
        String namespacedId = messageId.contains(":") ? messageId : ("minecraft:" + messageId);
        class_2960 location = class_2960.method_12829(namespacedId);
        return location != null ? location.toString() : messageId;
    }

}

==> sources_fancymenu_fabric_3.8.1_MC_1.21.10.jar: de/keksuccino/fancymenu/mixin/mixins/common/server/MixinLivingEntity.java <==
package de.keksuccino.fancymenu.mixin.mixins.common.server;

import de.keksuccino.fancymenu.networking.PacketHandler;
import de.keksuccino.fancymenu.networking.packets.entities.EntityEventPacket;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_2960;
import net.minecraft.class_3218;
import net.minecraft.class_7923;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_1309.class)
public class MixinLivingEntity {

    /** @reason Broadcast FancyMenu entity death events to connected clients. */
    @Inject(method = "die", at = @At("TAIL"))
    private void after_die_FancyMenu(@NotNull class_1282 damageSource, CallbackInfo info) {
        class_1309 self = (class_1309)(Object)this;
        if (!(self.method_73183() instanceof class_3218 serverLevel)) {
            return;
        }
        if (serverLevel.method_8503() == null) {
            return;
        }
        this.broadcastEntityDeath_FancyMenu(serverLevel, self, damageSource);
    }

    @Unique
    private void broadcastEntityDeath_FancyMenu(@NotNull class_3218 level, @NotNull class_1309 entity, @NotNull class_1282 damageSource) {
        EntityEventPacket packet = new EntityEventPacket();
        packet.event_type = EntityEventPacket.EntityEventType.DEATH;
        class_2960 entityKeyLocation = class_7923.field_41177.method_10221(entity.method_5864());
        packet.entity_key = (entityKeyLocation != null) ? entityKeyLocation.toString() : null;
        packet.entity_uuid = entity.method_5667().toString();
        packet.pos_x = entity.method_23317();
        packet.pos_y = entity.method_23318();
        packet.pos_z = entity.method_23321();
        class_2960 levelLocation = level.method_27983().method_29177();
        packet.level_identifier = (levelLocation != null) ? levelLocation.toString() : null;
        packet.damage_type = this.resolveDamageTypeKey_FancyMenu(damageSource);

        class_1297 killer = this.resolveKillerEntity_FancyMenu(entity, damageSource);
        if (killer != null) {
            packet.killer_name = killer.method_5476().getString();
            packet.killer_uuid = killer.method_5667().toString();
            class_2960 killerKeyLocation = class_7923.field_41177.method_10221(killer.method_5864());
            packet.killer_key = (killerKeyLocation != null) ? killerKeyLocation.toString() : null;
            if (packet.killer_key == null && killer instanceof net.minecraft.class_1657) {
                packet.killer_key = "minecraft:player";
            }
        } else {
            packet.killer_name = null;
            packet.killer_uuid = null;
            packet.killer_key = null;
        }

        PacketHandler.sendToAllFancyMenuClients(level.method_8503(), packet);
    }

    @Unique
    @Nullable
    private class_1297 resolveKillerEntity_FancyMenu(@NotNull class_1309 victim, @Nullable class_1282 damageSource) {
        if (damageSource != null) {
            class_1297 killer = damageSource.method_5529();
            if (killer != null) {
                return killer;
            }
            class_1297 directEntity = damageSource.method_5526();
            if (directEntity != null) {
                return directEntity;
            }
        }
        class_1309 killCredit = victim.method_6124();
        return killCredit != null ? killCredit : null;
    }

    @Unique
    @NotNull
    private String resolveDamageTypeKey_FancyMenu(@Nullable class_1282 damageSource) {
        if (damageSource == null) {
            return "unknown";
        }
        return damageSource.method_48793().method_40230()
                .map(key -> key.method_29177().toString())
                .orElse("unknown");
    }
}
